/********************************************************************** 
 Freeciv - Copyright (C) 2002 - The Freeciv Project
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/
#ifndef FC__PATH_FINDING_GB_H
#define FC__PATH_FINDING_GB_H

#include "map.h"
#include "unit.h"
#include "unittype.h"
#include "terrain.h"

#include "path_finding_common.h"

#undef pf_map
#undef pf_map_t
#undef pf_init
#undef pf_get_map
#undef pf_get_path
#undef pf_next
#undef pf_next_get_x
#undef pf_next_get_y
#undef pf_next_get_path
#undef pf_next_get_position
#undef pf_destroy_map

#define pf_map GB_pf_map
#define pf_map_t GB_pf_map_t
#define pf_init GB_pf_init
#define pf_get_map GB_pf_get_map
#define pf_get_path GB_pf_get_path
#define pf_next GB_pf_next
#define pf_next_get_x GB_pf_next_get_x
#define pf_next_get_y GB_pf_next_get_y
#define pf_next_get_path GB_pf_next_get_path
#define pf_next_get_position GB_pf_next_get_position
#define pf_destroy_map GB_pf_destroy_map
#define pf_print_path GB_pf_print_path


/*
 * Opaque types.
 */
typedef struct pf_map *pf_map_t;


/*
 * Called only once per executable start.
 */
void pf_init(void);

/*
 * Returns a map which can be used to query for paths or to iterate
 * over all paths. Does not perform any computations itself, just sets
 * everything up.
 */
pf_map_t pf_get_map(const struct pf_parameter *const parameter);

/*
 * Tries to find the best path in the given map to the given
 * destination position. The path will be written in the given struct
 * pf_path. The caller should test path->is_valid.
 */
void pf_get_path(pf_map_t pf_map, int end_x, int end_y,
		 struct pf_path *path);

/*
 * Saves the next best path in an internal buffer. This buffer can be
 * queried with pf_next_get_x, pf_next_get_y, pf_next_get_position and
 * pf_next_get_path. Returns FALSE if no more positions are available
 * in this map. Behavior is undefined after pf_get_path was called.
 */
bool pf_next(pf_map_t pf_map);

/*
 * Return the map position of the last position in the path which was
 * saved in the internal buffer.
 */
int pf_next_get_x(pf_map_t pf_map);
int pf_next_get_y(pf_map_t pf_map);

/*
 * Return the path which was saved in the internal buffer.
 */
bool pf_next_get_path(const pf_map_t pf_map, struct pf_path *path);

/*
 * Return the position of the last position in the path which was saved
 * in the internal buffer.
 */
void pf_next_get_position(pf_map_t pf_map, struct pf_position *pos);

/*
 * After usage the map should be destroyed.
 */
void pf_destroy_map(pf_map_t pf_map);

void pf_print_path(const struct pf_path *path);

 /*
  * Returns the last position of the given path.
  */
struct pf_position *pf_last_position(struct pf_path *path);

#endif
